SEXP_CONV_PATH "Goba"

open VecOps

let bitmap_offset = (-60, -60)

let bitmap = lazy (Bitmaps.bitmap (Bitmaps.move bitmap_offset (Bitmaps.colored (Bitmaps.auto_crop (Bitmaps.file "base"), (255, 255, 255)))))

let cmask = 
  let module B = Bitmaps in
    lazy 
      (B.mask_with_offset B.collision_mask_of_surface2 
	 (B.bitmap (B.move bitmap_offset (B.auto_crop (B.file "base-mask"))))) 

type init = { init_at		: float * float;
	      init_hp		: int;
	      init_color	: int * int * int }
with sexp

let max_hp = 30

let defaults = { init_at = (0.0, 0.0);
		 init_hp = max_hp;
		 init_color = (255, 255, 255);
	       }

class t id init0 =
  let init = Object.default_with init0 init_of_sexp in
object (self : 'a)
  inherit Object.t id as super

  val mutable hp = init.init_hp
  val mutable alive = true
  val mutable bitmap = fst (Base.health_frames (float init.init_hp /. float max_hp) (float init.init_hp /. float max_hp) init.init_color)
  val mutable bitmap_changed = false

  method get_state delay =
    Object.sexp_of_state sexp_of_init
      { Object.state_super = super#get_state delay;
	state_init = { init_at = init.init_at;
		       init_hp = hp;
		       init_color = init.init_color } }

  method set_state state delay =
    let state = Object.state_of_sexp init_of_sexp state in
      super#set_state state.Object.state_super delay;
      let init = state.Object.state_init in
	hp <- init.init_hp

  method set_speed _ = ()
  method set_location _ = ()

  method tick delta world =
    let died = 
      if hp <= 0 && alive then
	begin
	  alive <- false;
	  self#change;
	  true
	end
      else
	false
    in
      if bitmap_changed || died then
	begin
	  bitmap_changed <- false;
	  world#explode ~sound:true ~kind:ExplosionKind.Massive (init.init_at +|. (Random.float 15.0 -. 7.0, Random.float 15.0 -. 7.0)) (0.0, 0.0);
	end;

  method location = init.init_at
  method speed = 0.0, 0.0

  method render gfx flags at =
    Bitmaps.blit gfx at (Lazy.force bitmap)

  method cause_damage damage =
    let hp' = hp in
      hp <- max 0 (hp - damage);
      let bitmap', changed = Base.health_frames (float hp' /. float max_hp) (float hp /. float max_hp) init.init_color in
	bitmap <- bitmap';
	bitmap_changed <- bitmap_changed || changed;
	self#change

  method is_alive = alive

  method get_type = ObjectType.FakeBuilding

  method collision_area =
    Collision.carea_of_mask init.init_at (Lazy.force cmask)

  initializer
    (match init0 with 
       | Object.Init _ -> ()
       | Object.State (state0, age) -> self#set_state state0 age);
    self#change;
end

let has_space_for : Object.base -> (((float * float) * (float * float)) -> Object.base list) -> bool =
  fun obj objs ->
    let (region, _) = obj#collision_area in
      Collision.collisions obj (objs region) (GobaUtils.const true) = []
